@@ -346,6 +346,9 @@ def skip_function(name):
         return 1
     if name == "xmlFreeValidCtxt":
         return 1
+    if name == "xmlSchemaFreeValidCtxt":
+        return 1
+
 #
 # Those are skipped because the Const version is used of the bindings
 # instead.
@@ -744,6 +747,9 @@ classes_ancestor = {
     "inputBuffer": "ioReadWrapper",
     "parserCtxt": "parserCtxtCore",
     "xmlTextReader": "xmlTextReaderCore",
+    "ValidCtxt": "ValidCtxtCore",
+    "SchemaValidCtxt": "SchemaValidCtxtCore",
+    "relaxNgValidCtxt": "relaxNgValidCtxtCore",
 }
 classes_destructors = {
     "parserCtxt": "xmlFreeParserCtxt",
